"""Single-connection Postgres client with high-level wrappers for operations."""

from dataclasses import dataclass
from typing import Any, Iterable

import trio


def _channel_sender(method):
	async def wrapped(self, resp_channel, *args, **kwargs):
		ret = await method(self, *args, **kwargs)
		async with resp_channel:
			await resp_channel.send(ret)

	return wrapped


@dataclass
class Client:
	"""High-level wrappers for DB operations."""
	conn: Any

	@_channel_sender
	async def get_auth_tokens(self, auth_ids: set[int]) -> dict[int, str]:
		results = await self.conn.fetch(
			"""
				select
					distinct on (id)
					id, access_token
				from
					public.reddit_app_authorization
					join unnest($1::integer[]) as request_id on id = request_id
				where expires > current_timestamp
			""",
			list(auth_ids),
		)
		return {r["id"]: r["access_token"] for r in results}


class Messenger:
	def __init__(self, client: Client):
		self._client = client
		(self._request_tx, self._request_rx) = trio.open_memory_channel(0)

	async def do(self, method_name: str, args: Iterable) -> Any:
		"""This is run by consumers of the DB wrapper."""
		method = getattr(self._client, method_name)
		(resp_tx, resp_rx) = trio.open_memory_channel(0)
		coro = method(resp_tx, *args)
		await self._request_tx.send(coro)
		async with resp_rx:
			return await resp_rx.receive()

	async def db_client_impl(self):
		"""This is the DB client Trio task."""
		async with self._client.conn, self._request_rx:
			async for coro in self._request_rx:
				await coro
